#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Core RPC code.
#
# Heavily modified version of the python CLI
# https://plugins.atlassian.com/plugin/details/10751

import types

from datetime import datetime
from suds.sax.date import DateTime as sudsdatetime

from jira import settings
from jira.rpc_utils import (
    change_status, rpc_init, setup_logging, get_project_choices, get_field_id,
    JiraException, RPCException,
)

log = setup_logging()


class Project(object):
    """Class representing a project in JIRA.

    This holds all methods for accessing, updating, and creating content in
    JIRA via SOAP. Project-level seemed like a good entry point since that's
    the base unit of aggregation in JIRA. There is no explicit class
    representing an issue (or anything else); anything requiring access to an
    issue, eg: `add_comment` just takes an issue ID as a parameter.

    Usage:

    >>> from jira.rpc import Project
    >>> p = Project('TEST')
    >>> result = p.create_issue('Test issue using all available defaults')
    >>> p.add_comment(result['key'], 'First comment on the new issue!')

    """
    def __init__(self, project_id):
        self.env = rpc_init(log)

        if settings.get('TESTING'):
            log.debug('Overriding JIRA project from %s to TEST' % project_id)
            project_id = settings.get('TEST_PROJECT')

        projects = get_project_choices(self.env)
        if project_id not in projects:
            raise JiraException("%s isn't a valid project ID." % project_id)

        self.project_id = project_id

        client = self.env['client']
        auth = self.env['auth']
        server_info = client.service.getServerInfo(auth)
        log.debug("Server info: %s" % server_info)

        if 'auth' not in self.env or not self.env['auth']:
            log.critical("Error acquiring auth token")
            raise RPCException(self.env)

        if 'client' not in self.env or not self.env['client']:
            log.critical("Error acquiring client")
            raise RPCException(self.env)

    def create_issue(self, summary, assignee=None, priority='Major',
                     issue_type='Task', description=None, estimate=None,
                     parent_id=None, affects_versions=None, fix_versions=None):
        """Create an issue in JIRA. Only requires a summary.

        To create sub-tasks, simply pass in the parent ID as ``parent_id``.

        """
        client = self.env['client']
        auth = self.env['auth']

        try:
            self.env['fieldnames'] = client.service.getCustomFields(auth)
        except Exception:
            # In case we don't have permission to get the fields
            self.env['fieldnames'] = [{'id':-1, 'name':'unavailable'}]

        priority_id = 0
        for i, v in enumerate(self.env['priorities']):
            if v['name'] == priority:
                priority_id = v['id']

        if priority_id == 0:
            raise JiraException('Unknown priority: ' + priority)

        issue_type_id = 0
        if parent_id:
            for i, v in enumerate(self.env['subtypes']):
                if v['name'] == issue_type:
                    issue_type_id = v['id']
        else:
            for i, v in enumerate(self.env['types']):
                if v['name'] == issue_type:
                    issue_type_id = v['id']

        if issue_type == 0:
            log.error('Unknown issue type: %s' % issue_type)
            log.error(
                'Known issue types: %s' %
                ", ".join(map(lambda i: i['name'], self.env['types']))
            )
            log.error(
                'Known subtask issue types: %s' %
                ", ".join(map(lambda i: i['name'], self.env['subtypes']))
            )

            raise JiraException("Unknown issue type %s" % issue_type)

        remote_issue = {
            "project": self.project_id,
            "type": issue_type_id,
            "summary": summary,
            "priority": priority_id,
            "description": description,
            "assignee": assignee,
        }

        if affects_versions:
            remote_issue['affectsVersions'] = affects_versions

        if fix_versions:
            remote_issue['fixVersions'] = fix_versions

        if not parent_id:
            return client.service.createIssue(auth, remote_issue)
        else:
            return client.service.createIssueWithParent(
                auth, remote_issue, parent_id,
            )

    def add_comment(self, issue_id, comment):
        if not issue_id or not comment:
            raise JiraException("Issue ID and comment text are both required.")

        client = self.env['client']
        auth = self.env['auth']
        try:
            return client.service.addComment(
                auth, issue_id, dict(body=comment))
        except Exception, e:
            log.error(e)
            raise

    def close_issue(self, issue_id):
        action = 'Close Issue'
        return change_status(issue_id, action, self.env, log)

    def reopen_issue(self, issue_id):
        action = 'Reopen Issue'
        return change_status(issue_id, action, self.env, log)

    def resolve_issue(self, issue_id):
        action = 'Resolve Issue'
        return change_status(self.issue_id, action, self.env, log)

    def update_issue_attribute(self, issue_id, field, value):
        if type(value) in types.StringTypes:
            value = value.split(',')
        client = self.env['client']
        auth = self.env['auth']

        fields = [
            'key', 'summary', 'reporter', 'assignee', 'description',
            'environment', 'project', 'type', 'status', 'priority',
            'resolution', 'duedate', 'originalEstimate', 'timeLogged',
            'votes',
        ]
        if field not in fields and not field.startswith("customfield_"):
            try:
                self.env['fieldnames'] = client.service.getCustomFields(auth)
            except:
                # In case we don't have permission to get the fields
                self.env['fieldnames'] = [{'id': -1, 'name': 'unavailable'}]

            cn = get_field_id(field, self.env['fieldnames'])
            if cn == "unknown":
                raise JiraException(
                    "Field name '%s' not found in %s" %
                         (field, fields + self.env['fieldnames'])
                )
            self.field = cn
        return client.service.updateIssue(
            auth, issue_id, [{'id': field, 'values': value}]
        )

    def add_version(self, version_name, release_date):
        release_date = datetime.strptime(release_date, '%Y-%m-%d')
        client = self.env['client']
        auth = self.env['auth']

        try:
            # https://fedorahosted.org/suds/ticket/219
            # seems to be fixed in 0.3.9
            dateObj = sudsdatetime(release_date)
            version = client.factory.create('ns0:RemoteVersion')
            version.name = version_name
            version.releaseDate = dateObj
            version.archived = False
            version.released = False

            return client.service.addVersion(auth, self.project_id, version)
        except Exception, e:
            log.error(e)
            raise JiraException(e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sys

from suds.client import Client

from jira import settings


class JiraException(Exception):
    pass


class RPCException(Exception):
    def __init__(self, env, *args, **kwargs):
        self.message = 'env: %s' % str(env)
        super(self, RPCException).__init__(*args, **kwargs)


def setup_logging(loglevel=logging.INFO):
    """Set up logging, by default just echo to stdout."""
    logger = logging.getLogger()
    logger.setLevel(loglevel)
    ch = logging.StreamHandler(sys.stdout)
    ch.setLevel(loglevel)

    formatter = logging.Formatter("%(message)s")
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    return logger


def get_project_choices(env):
    """Concatentate the names of values in a structure."""
    results = ""
    first_choice = True
    for i, v in enumerate(env['projects']):
        if first_choice:
            results = "%s (%s)" % (v['name'], v['key'])
            first_choice = False
        else:
            results += ", %s (%s)" % (v['name'], v['key'])
    return results


def get_field_id(name, fields):
    result = 'unknown'

    matches = [x['id'] for x in fields if x['name'] == name]
    if len(matches):
        result = matches[0]

    return result


def change_status(issue_id, action, env, logger, **kwargs):
    """Generic function for changing the status of an issue."""
    client = env['client']
    auth = env['auth']

    try:
        actions = client.service.getAvailableActions(auth, issue_id)
    except Exception, e:
        logger.error(e)

    action_id = ''
    for i, v in enumerate(actions):
        if v['name'] == action:
            action_id = v['id']
            break

    if action_id == '':
        logger.error('Unable to perform action: ' + action)
        logger.error('Choices are: ')
        for i, v in enumerate(actions):
            logger.info(v['name'] + ' (' + v['id'] + ')')
        return 1

    if 'resolution' in kwargs:
        resolution = kwargs['resolution']
    else:
        resolution = '1'

    if 'timetracking' in kwargs:
        timetracking = kwargs['timetracking']
    else:
        timetracking = '0m'

    try:
        return client.service.progressWorkflowAction(
            auth, issue_id, action_id,
            [
                {"id": "assignee", "values": [env['jirauser']]},
                {"id": "resolution", "values": [resolution]},
                {"id": "timetracking", "values": [timetracking]},
            ]
        )
    except Exception, e:
        logger.error(e)


def get_client(logger):
    WSDL_URL = settings.get('WSDL_URL')

    try:
        logger.debug('Attempting to connect to the server: ' + WSDL_URL)

        client = Client(WSDL_URL)
        logger.debug('Connected to the server')
        logger.debug(client)
        return client
    except Exception, e:
        logger.error('Failed to connect to JIRA (%s): %s' % (WSDL_URL, e))
        raise


def rpc_init(logger):
    env = dict(
        jirauser=settings.get('USERNAME'),
    )

    authorized = False
    while not authorized:
        try:
            logger.debug('Starting authorization')

            client = get_client(logger)
            auth = client.service.login(
                env['jirauser'], settings.get('PASSWORD'),
            )

            logger.debug('Using auth to get types')
            env['types'] = client.service.getIssueTypes(auth)

            logger.debug('Using auth to get subtask types')
            env['subtypes'] = client.service.getSubTaskIssueTypes(auth)

            logger.debug('Using auth to get statuses')
            env['statuses'] = client.service.getStatuses(auth)

            logger.debug('Using auth to get priorities')
            env['priorities'] = client.service.getPriorities(auth)

            logger.debug('Using auth to get resolutions')
            env['resolutions'] = client.service.getResolutions(auth)

            logger.debug('Using auth to get projects')
            env['projects'] = client.service.getProjectsNoSchemes(auth)

            authorized = True

            env['client'] = client
            env['auth'] = auth
        except Exception, e:
            logger.error('Error authenticating with JIRA')
            logger.exception(e)
            raise

    return env



import os

__all__ = ['settings']

__title__ = 'jira-rpc'
__version__ = '0.1'
__description__ = 'Python client for JIRA SOAP API'
__url__ = 'https://github.com/Maplecroft/jira-rpc'
__author__ = 'James Rutherford'
__licence__ = 'MIT'
__copyright__ = 'Copyright 2012 Maplecroft'


class ConfigurationException(Exception):
    pass


CFG_PATH = os.path.join(
    os.path.dirname(__file__), 'config.ini'
)

settings = dict()

wsdl_url = username = password = None
testing = False
test_project = 'TEST'

if os.path.exists(CFG_PATH):
    from ConfigParser import SafeConfigParser

    parser = SafeConfigParser()
    parser.read(CFG_PATH)

    wsdl_url = parser.get('main', 'wsdl_url')
    testing = parser.getboolean('main', 'testing')
    test_project = parser.get('main', 'test_project')
    username = parser.get('auth', 'username')
    password = parser.get('auth', 'password')
else:
    try:
        from django.conf import settings as django_settings
        try:
            testing = django_settings.JIRA_TESTING
        except AttributeError:
            pass

        try:
            test_project = django_settings.JIRA_TEST_PROJECT
        except AttributeError:
            pass

        wsdl_url = django_settings.JIRA_WSDL_URL
        username = django_settings.JIRA_RPC_USER
        password = django_settings.JIRA_RPC_PASS
    except:
        pass

if not (wsdl_url and username and password):
    raise ConfigurationException(
        "Please make sure you have configured the WSDL URL, username, and "
        "password either in your Django settings or in a config.ini file."
    )

settings['WSDL_URL'] = wsdl_url
settings['USERNAME'] = username
settings['PASSWORD'] = password
settings['TESTING'] = testing
settings['TEST_PROJECT'] = test_project



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

PACKAGES = ['jira']


def get_init_val(val, packages=PACKAGES):
    pkg_init = "%s/__init__.py" % PACKAGES[0]
    value = '__%s__' % val
    fn = open(pkg_init)
    for line in fn.readlines():
        if line.startswith(value):
            return line.split('=')[1].strip().strip("'")


setup(
    name=get_init_val('title'),
    version=get_init_val('version'),
    provides=['jira'],
    description=get_init_val('description'),
    long_description=open('README.rst').read(),
    author=get_init_val('author'),
    author_email='james.rutherford@maplecroft.com',
    url=get_init_val('url'),
    keywords=['jira', 'rpc'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        ],
    license=get_init_val('license'),
    packages=PACKAGES,
    test_suite='tests',
)
